/*
 * FILE --- k_fltr.hh
 *
 * PURPOSE --- Header file for a class that implements a Kalman Filter
 * algorithm for state estimation for Multiple-Input-Multiple-Output (MIMO) 
 * plants.
 * The user specifies the dimensions of the input, state, and output spaces,
 * as well as initial state estimate vector and initial state covariance 
 * matrix, and noise covariance matrices.
 * 
 * The system assumed to be in the form
 * x(k+1) = Ax(k) + Bu(k) + v_1(k)
 * y(k) = Cx(k) + v_2(k)
 * where v_1 and v_2 are zero mean (possibly gaussian) random noises
 * v_1 = system noise with autocovariance Q = E{v_1^Tv_1}
 * v_2 = output noise with autocovariance R = E{v_2^Tv_2}
 * S = E{v_1^Tv_2} is the covariance b/n system and output noises
 *
 * The Kalman filter variables are
 * xhat = the vector of estimeates of the state
 * K = Gain matrix for the filter
 * Sigma = covariance matrix for the state
 *
 * This utility uses the matrix tool for matrix maltiplications.
 *
 */

// Prevent Multiple Inclusion
#ifndef K_FLTR_HH
#define K_FLTR_HH

// Include Files
#include <stdlib.h>
#include "matrix.hh"

// Local Definitions
#define K_FLTR_NO_ERROR 0
#define K_FLTR_MEM_ERROR 1

class K_FLTR
{
public:
  
  // Constructor
  K_FLTR(int _num_of_states, int _num_of_inputs = 1, int num_of_outputs = 1);
  
  // Destructor
  ~K_FLTR();
  
  // Set or return the state of the filter
  void set_state(const double *_xhat); // Set state xhat to given value 
  void get_state(double *_xhat); // Return the current value of xhat
  
  // Set the covarinace matrix Sigma to given value
  void set_cov_Sigma(const double **_Sigma); 
  // Set the covarinace matrix Sigma to a diagonal matrix
  void set_cov_Sigma(const double *_diagonal); 
  // Set the covarinace matrix Sigma to _val*identity
  void set_cov_Sigma(double _val); 
  // Return the current value of the covarinace matrix Sigma
  void get_cov_Sigma(double **_Sigma); 
  
  // Set the system matrix A to the specified value
  void set_sys_matrix_A(const double **_A); 
  // Return the current value of the system matrix A
  void get_sys_matrix_A(double **_A); 
  
  // Set the system matrix B to the specified value
  void set_sys_matrix_B(const double **_B); 
  // Set the system matrix B to the specified value
  void set_sys_matrix_B(const double *_B); 
  // Return the current value of the system matrix B
  void get_sys_matrix_B(double **_B); 
  
  // Set the system matrix C to the specified value
  void set_sys_matrix_C(const double **_C); 
  // Set the system matrix C to the specified value
  void set_sys_matrix_C(const double *_C); 
  // Return the current value of the system matrix C
  void get_sys_matrix_C(double **_C); 

  // Set the system noise autocovariance matrix Q
  void set_sys_noise_autocov(const double **_Q); 
  // Set the system noise autocovariance matrix Q to a diagonal matrix
  void set_sys_noise_autocov(const double *_diagonal); 
  // Set the system noise autocovariance matrix Q to _val*identity
  void set_sys_noise_autocov(double _val); 
  // Return the current value of the system noise autocovariance matrix Q
  void get_sys_noise_autocov(double **_Q); 

  // Set the output (measurement) noise autocovariance matrix R
  void set_out_noise_autocov(const double **_R); // multi-output 
  // Set the output noise autocovariance matrix R to a diagonal matrix
  void set_out_noise_autocov(const double *_diagonal); 
  // Set the output noise autocovariance matrix R to _val*identity
  void set_out_noise_autocov(double _val); 
  // Return the current value of the output noise autocovariance matrix R
  void get_out_noise_autocov(double **_R); 

  // Set the covariance matrix S b/n system and output noises
  void set_noise_cov(const double **_S); 
  // Set the covariance matrix S b/n system and output noises
  void set_noise_cov(const double *_S); 
  // Return the current value of the noise covariance matrix S
  void get_noise_cov(double **_S); 

  
  // Calculate the new estimate 
  void filter(const double *plant_output, 
	      const double *ctrl_input,
	      double *_xhat);

  int k_fltr_err;  // Variable to hold the error condition
  
private:
  // # of inputs, outputs, and states
  int num_of_inputs;
  int num_of_outputs;
  int num_of_states;
  
  // The state vector, the gain matrix, and the covariance matrix
  matrix *xhat;
  matrix *K;
  matrix *Sigma;
  
  // The system matrices
  matrix *A;
  matrix *B;
  matrix *C;

  // Noise covariance matrices
  matrix *Q; // System noise autovcovariance
  matrix *R; // Measurement noise autocovariance
  matrix *S; // Covariance of the system and measurement noise

  // Convaniance variables
  matrix *u; // Control vector
  matrix *y; // Output vector
  double output_error;
  double temp;
};

#endif // K_FLTR_HH
