/*
 * FILE --- k_fltr_ex.cc 
 *
 * PURPOSE --- Main source file that provides an example usage of the
 * Kalman Filter algorithm for state estimation for 
 * Multiple-Input-Multiple-Output (MIMO) plants.
 * The user specifies the dimensions of the input, state, and output spaces,
 * as well as initial state estimate vector and initial state covariance 
 * matrix, and noise covariance matrices.
 * 
 * The system assumed to be in the form
 * x(k+1) = Ax(k) + Bu(k) + v_1(k)
 * y(k) = Cx(k) + v_2(k)
 * where v_1 and v_2 are zero mean (possibly gaussian) random noises
 * v_1 = system noise with autocovariance Q = E{v_1^Tv_1}
 * v_2 = output noise with autocovariance R = E{v_2^Tv_2}
 * S = E{v_1^Tv_2} is the covariance b/n system and output noises
 *
 * The Kalman filter variables are
 * xhat = the vector of estimeates of the state
 * K = Gain matrix for the filter
 * Sigma = covariance matrix for the state
 *
 */

// Include Files
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "k_fltr.hh"
#include "erfinv.hh"

void main(void) 
{
  // Implements example 
  // x(k+1) = Ax(k) + Bu(k) + v_1(k)
  // y(k) = Cx(k) + v_2(k)
  // with A = [1.0 0.01; 0.0 1.0]; B = [0.00005; 0.01]; C = [1.0 0.0]
  // The covariance matrices of the noises are assumed to be
  // Q = [0.0 0.0; 0.0 0.01]; R = 0.1; S = [0.0; 0.0]
  // We intialize the Kalman Filter with
  // x(0) = [0.0; 0.0]; and Sigma(0) = [0.1 0.0; 0.0 0.1]

  // The order of the system
  int n_states = 2, n_inputs = 1, n_outputs = 1;

  // The Kalman Filter
  K_FLTR my_filter(n_states, n_inputs, n_outputs);

  // Kalman Filter state and its covariance
  double xhat[2] = {0.0, 0.0};
  double Sigma0 = 0.1;

  // Noise variances
  double Q[2] = {0.0, 0.01};
  double R = 0.1;
  double S[2] = {0.0, 0.0};

  // Actual system 
  double x[2] = {5.0, -5.0};
  double **A;
  double B[2] = {0.00005, 0.01};
  double C[2] = {1.0, 0.0};
  double u = 1.0, y, tmp;
  double w_1 = 0.0, w_2 = 0.0, v = 0.0;
 
  // Maximum # of iterations
  int max_iterations = 200;

  FILE *fp;  // To save data

  A = new double*[2];
  A[0] = new double[2];
  A[1] = new double[2];
  A[0][0] = 1.0; A[0][1] = 0.01; 
  A[1][0] = 0.0; A[1][1] = 1.0; 

  // Initialise the KF
  my_filter.set_sys_matrix_A(A);
  my_filter.set_sys_matrix_B(B);
  my_filter.set_sys_matrix_C(C);
  my_filter.set_state(xhat);
  my_filter.set_cov_Sigma(Sigma0);  // Sigma = 0.1*Identity
  my_filter.set_sys_noise_autocov(Q); // autocov = diagonal with given values
  my_filter.set_out_noise_autocov(R); // autocov = R
  my_filter.set_noise_cov(S); // cov = S

  // Open the file
  if (!(fp = fopen("k_fltr_out.dat","w"))) {
    printf("Can't open file.\n");
    exit(1);
  }
  
  // Simulate for max_itearions
  for(int ii=0; ii < max_iterations; ii++)
    {
      // Calculate current system inputs and noise
      u = 1.0;
      w_1 = 0.0;
      w_2 = sqrt(Q[1])*erfinv((2.0*rand()/(RAND_MAX+1.0)) - 1.0);
      v = sqrt(R)*erfinv((2.0*rand()/(RAND_MAX+1.0)) - 1.0);

      // Calculate the system dynamics
      tmp = A[0][0]*x[0] + A[0][1]*x[1] + B[0]*u + w_1;
      x[1] = A[1][0]*x[0] + A[1][1]*x[1] + B[1]*u + w_2;
      x[0] = tmp;
      y = C[0]*x[0] + C[1]*x[1] + v;

      // Calculate the new estimate
      my_filter.filter(&y, &u, xhat);

      // Save the data
      fprintf(fp,"%f %f %f %f %f %f \n", y, u, x[0], x[1], xhat[0], xhat[1]);
    } 
  
  fclose(fp);
}

